@@ -30,11 +30,13 @@ settings::settings()
 	defaul["mandatory_dn"] = "";
 	defaul["explicit_dn"] = "C,ST,L,O,OU,CN,emailAddress";
 	defaul["string_opt"] = "MASK:0x2002";
-	defaul["workingdir"] = QDir::toNativeSeparators(QDir::currentPath());
+	defaul["workingdir"] = getHomeDir();
 	defaul["default_hash"] = hashBox::getDefault();
 	defaul["ical_expiry"] = "1W";
 	defaul["cert_expiry"] = "80%";
 	defaul["serial_len"] = "64";
+	defaul["fp_separator"] = ":";
+	defaul["fp_digits"] = "2";
 
 	hostspecific << "pkcs11path" << "workingdir" << "mw_geometry";
 
@@ -88,10 +90,12 @@ void settings::load_settings()
 	XSqlQuery q("SELECT key_, value FROM settings");
 	while (q.next()) {
 		QString key = q.value(0).toString().simplified();
-		QString value = q.value(1).toString().simplified();
+		QString value = q.value(1).toString();
 		QStringList l = key.split(":");
 		if (l.size() == 2 && l[1] != hostId())
 			continue;	// Skip key with non-matching host ID
+		if (l[0] == "workingdir" && !QDir(value).exists())
+			continue;	// Skip non-existing working-dir
 		db_keys << key;		// Key with host ID
 		setAction(l[0], value);	// Key without host ID
 	}
@@ -108,15 +112,16 @@ void settings::set(QString key, QString value)
 {
 	XSqlQuery q;
 	load_settings();
+	QString origkey = key;
 
 	if (key == "workingdir") {
-		if (portable_app() || !QDir(value).exists())
+		if (!QDir(value).exists())
 			return;
-		value = QDir::toNativeSeparators(value);
+		value = relativePath(value);
 	}
 	if (hostspecific.contains(key))
 		key += QString(":%1").arg(hostId());
-	if (db_keys.contains(key) && values[key] == value)
+	if (db_keys.contains(key) && values[origkey] == value)
 		return;
 
 	Transaction;
@@ -132,6 +137,6 @@ void settings::set(QString key, QString value)
 	q.bindValue(0, value);
 	q.bindValue(1, key);
 	q.exec();
-	setAction(key, value);
+	setAction(origkey, value);
 	TransCommit();
 }
