@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2017 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2020 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -25,14 +25,17 @@
 
 QSqlError MainWindow::initSqlDB()
 {
-	QStringList schemas[6];
+#define MAX_SCHEMAS 7
+#define SCHEMA_VERSION "7"
+
+	QStringList schemas[MAX_SCHEMAS];
 
 #include "database_schema.cpp"
 
 	XSqlQuery q;
 	QSqlDatabase db = QSqlDatabase::database();
 	QStringList tables;
-	unsigned int i;
+
 	if (!db.isOpen())
 		return QSqlError();
 
@@ -40,7 +43,10 @@ QSqlError MainWindow::initSqlDB()
 	if (!TransBegin())
 		return db.lastError();
 
-	for (i = XSqlQuery::schemaVersion(); i < ARRAY_SIZE(schemas); i++) {
+	for (;;) {
+		unsigned int i = XSqlQuery::schemaVersion();
+		if (i >= ARRAY_SIZE(schemas))
+			break;
 		foreach(QString sql, schemas[i]) {
 			qDebug("EXEC[%d]: '%s'", i, CCHAR(sql));
 			if (!q.exec(sql) || q.lastError().isValid()) {
@@ -49,6 +55,7 @@ QSqlError MainWindow::initSqlDB()
 			}
 		}
 	}
+
 	TransCommit();
 	return QSqlError();
 }
@@ -267,6 +274,8 @@ int MainWindow::init_database(QString dbName)
 			ret = initPass(dbName);
 			if (ret == 2)
 				return ret;
+			if (ret == 0 && Settings["pwhash"].empty())
+				return 2;
 			ret = 0;
 		}
 		keys = new db_key(this);
@@ -274,6 +283,8 @@ int MainWindow::init_database(QString dbName)
 		certs = new db_x509(this);
 		temps = new db_temp(this);
 		crls = new db_crl(this);
+		check_oom(keys && reqs && certs && temps && crls);
+		models << keys << reqs << certs << temps << crls;
 	}
 	catch (errorEx &err) {
 		Error(err);
@@ -325,9 +336,42 @@ int MainWindow::init_database(QString dbName)
 	}
 	dbindex->setText(tr("Database") + ": " + dbName);
 	currentDB = dbName;
+	dbTimer = startTimer(1500);
 	return ret;
 }
 
+void MainWindow::timerEvent(QTimerEvent *event)
+{
+	quint64 stamp;
+	if (event->timerId() != dbTimer)
+		return;
+	XSqlQuery q;
+	SQL_PREPARE(q, "SELECT MAX(stamp) from items");
+	q.exec();
+	if (!q.first())
+		return;
+	stamp = q.value(0).toULongLong();
+	q.finish();
+	qDebug() << "Stamp" << stamp
+		 << "DatabaseStamp" << DbTransaction::DatabaseStamp;
+
+	if (stamp > DbTransaction::DatabaseStamp) {
+		SQL_PREPARE(q, "SELECT DISTINCT type FROM items WHERE stamp=?");
+		q.bindValue(0, stamp);
+		q.exec();
+
+		QList<enum pki_type> typelist;
+		while (q.next())
+			typelist << (enum pki_type)q.value(0).toInt();
+
+		q.finish();
+		qDebug() << "CHANGED" << typelist;
+		foreach(db_base *model, models)
+			model->reloadContainer(typelist);
+	}
+	DbTransaction::DatabaseStamp = stamp;
+}
+
 void MainWindow::dump_database()
 {
 	QString dirname = QFileDialog::getExistingDirectory(
@@ -337,7 +381,7 @@ void MainWindow::dump_database()
 		return;
 
 	QDir d(dirname);
-	if ( ! d.exists() && !d.mkdir(dirname)) {
+	if (!d.exists() && !d.mkdir(dirname)) {
 		errorEx err("Could not create '" + dirname + "'");
 		MainWindow::Error(err);
 		return;
@@ -345,11 +389,8 @@ void MainWindow::dump_database()
 
 	qDebug() << "Dumping to" << dirname;
 	try {
-		keys->dump(dirname);
-		certs->dump(dirname);
-		temps->dump(dirname);
-		crls->dump(dirname);
-		reqs->dump(dirname);
+		foreach(db_base *model, models)
+			model->dump(dirname);
 	}
 	catch (errorEx &err) {
 		MainWindow::Error(err);
@@ -422,11 +463,7 @@ int MainWindow::open_default_db()
 
 void MainWindow::default_database()
 {
-	if (portable_app())
-		return;
-
 	QFile file(defaultdb());
-	QFileInfo fi(currentDB);
 
 	if (currentDB.isEmpty()) {
 		file.remove();
@@ -438,7 +475,7 @@ void MainWindow::default_database()
 		if (OpenDb::isRemoteDB(currentDB))
 			ba = filename2bytearray(currentDB);
 		else
-			ba = filename2bytearray(fi.canonicalFilePath());
+			ba = filename2bytearray(relativePath(currentDB));
 		ba += '\n';
 		file.write(ba);
 		/* write() failed? Harmless. Only inconvenient */
@@ -464,6 +501,7 @@ void MainWindow::close_database()
 		Settings.clear();
 		return;
 	}
+	killTimer(dbTimer);
 	qDebug("Closing database: %s", QString2filename(currentDB));
 	Settings["mw_geometry"] = QString("%1,%2,%3")
 			.arg(size().width())
@@ -480,16 +518,8 @@ void MainWindow::close_database()
 	tempView->setModel();
 	crlView->setModel();
 
-	if (crls)
-		delete(crls);
-	if (reqs)
-		delete(reqs);
-	if (certs)
-		delete(certs);
-	if (temps)
-		delete(temps);
-	if (keys)
-		delete(keys);
+	qDeleteAll(models.begin(), models.end());
+	models.clear();
 
 	db_base::flushLookup();
 	reqs = NULL;
@@ -551,6 +581,9 @@ void MainWindow::update_history(QString fname)
 	QFile file;
 	int pos;
 
+	if (!OpenDb::isRemoteDB(fname))
+		fname = relativePath(fname);
+
 	pos = history.indexOf(fname);
 	if (pos == 0)
 		return; /* no changes */
@@ -563,9 +596,6 @@ void MainWindow::update_history(QString fname)
 
 	update_history_menu();
 
-	if (portable_app())
-		return;
-
 	file.setFileName(dbhistory());
 	if (!file.open(QIODevice::ReadWrite))
 		return;
