@@ -9,6 +9,7 @@
 //#define MDEBUG
 #include "MainWindow.h"
 #include "ImportMulti.h"
+#include "dhgen.h"
 #include "lib/Passwd.h"
 #include "lib/entropy.h"
 
@@ -25,6 +26,7 @@
 #include <QStatusBar>
 #include <QList>
 #include <QTimer>
+#include <QThread>
 #include <QMimeData>
 #include <QInputDialog>
 
@@ -68,11 +70,8 @@ void MainWindow::enableTokenMenu(bool enable)
 
 void MainWindow::load_engine()
 {
-	try {
-		pkcs11::load_libs(Settings["pkcs11path"], false);
-	} catch (errorEx &err) {
-		Error(err);
-	}
+	pkcs11::reload_libs(Settings["pkcs11path"]);
+	//	Error(err);
 	enableTokenMenu(pkcs11::loaded());
 }
 
@@ -155,6 +154,12 @@ MainWindow::MainWindow(QWidget *parent)
 	tempView->setMainwin(this, searchEdit);
 	crlView->setMainwin(this, searchEdit);
 	keys = NULL; reqs = NULL; certs = NULL; temps = NULL; crls = NULL;
+
+	dhgen = NULL;
+	dhgenBar = new QProgressBar();
+	check_oom(dhgenBar);
+	dhgenBar->setMinimum(0);
+	dhgenBar->setMaximum(0);
 }
 
 void MainWindow::dropEvent(QDropEvent *event)
@@ -380,7 +385,7 @@ bool MainWindow::pastePem(QString text, bool silent)
 	try {
 		pem = new pki_multi();
 		dlgi = new ImportMulti(this);
-		pem->fromPEMbyteArray(pemdata, QString("paste"));
+		pem->fromPEMbyteArray(pemdata, QString());
 		success = pem->count() != 0;
 		dlgi->addItem(pem);
 		pem = NULL;
@@ -597,6 +602,13 @@ MainWindow::~MainWindow()
 
 void MainWindow::closeEvent(QCloseEvent *e)
 {
+	if (dhgen) {
+		if (!XCA_YESNO("Abort Diffie-Hellmann parameter generation?")){
+			e->ignore();
+			return;
+		}
+		dhgen->terminate();
+	}
 	if (resolver) {
 		delete resolver;
 	}
@@ -813,12 +825,12 @@ pki_multi *MainWindow::probeAnything(QString file, int *ret)
 	} catch (errorEx &err) {
 		Error(err);
 	}
-	if (!pki->count()) {
+	if (pki && !pki->count()) {
 		delete pki;
 		pki = NULL;
-		if (ret)
-			*ret = 1;
 	}
+	if (!pki && ret)
+		*ret = 1;
 	return pki;
 }
 
@@ -851,20 +863,32 @@ int MainWindow::exportIndex(QString fname, bool hierarchy)
 	return 0;
 }
 
+void MainWindow::generateDHparamDone()
+{
+	statusBar()->removeWidget(dhgenBar);
+	errorEx e(dhgen->error);
+	if (e.isEmpty())
+		XCA_INFO(tr("Diffie-Hellman parameters saved as: %1")
+			.arg(dhgen->filename()));
+	else
+		Error(e);
+	dhgen->deleteLater();
+	dhgen = NULL;
+}
+
 void MainWindow::generateDHparam()
 {
-	DH *dh = NULL;
-	FILE *fp = NULL;
-	QProgressBar *bar = NULL;
 	bool ok;
-	int num = QInputDialog::getDouble(this, XCA_TITLE, tr("Diffie-Hellman parameters are needed for different applications, but not handled by XCA.\nPlease enter the DH parameter bits"),
-		1024, 1024, 4096, 0, &ok);
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-	BN_GENCB *cb = NULL;
-#endif
+	int bits;
 
+	if (dhgen)
+		return;
+
+	bits = QInputDialog::getDouble(this, XCA_TITLE, tr("Diffie-Hellman parameters are needed for different applications, but not handled by XCA.\nPlease enter the DH parameter bits"),
+		1024, 1024, 4096, 0, &ok);
 	if (!ok)
 		return;
+
 	/*
 	 * 1024:   6 sec
 	 * 2048:  38 sec
@@ -873,67 +897,33 @@ void MainWindow::generateDHparam()
 
 	Entropy::seed_rng();
 	try {
-		QStatusBar *status = statusBar();
-		bar = new QProgressBar();
-		check_oom(bar);
-		bar->setMinimum(0);
-		bar->setMaximum(100);
-		status->addPermanentWidget(bar, 1);
-
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-		cb = BN_GENCB_new();
-		if (!cb)
-			throw errorEx("");
-		BN_GENCB_set_old(cb, inc_progress_bar, bar);
-		dh = DH_new();
-		DH_generate_parameters_ex(dh, num, 2, cb);
-#else
-		dh = DH_generate_parameters(num, 2, inc_progress_bar, bar);
-#endif
-
-		status->removeWidget(bar);
-		openssl_error();
-
-		QString fname = QString("%1/dh%2.pem").arg(homedir).arg(num);
+		QString fname = QString("%1/dh%2.pem").arg(homedir).arg(bits);
 		fname = QFileDialog::getSaveFileName(this, QString(),
 			fname, tr("All files ( * )"), NULL);
 		if (fname == "")
 			throw errorEx("");
-		fp = fopen_write(fname);
-		if (fp == NULL) {
-			throw errorEx(tr("Error opening file: '%1': %2").
-				arg(fname).arg(strerror(errno)));
-		}
-		PEM_write_DHparams(fp, dh);
-		openssl_error();
+		dhgen = new DHgen(fname, bits);
+		check_oom(dhgen);
+		statusBar()->addPermanentWidget(dhgenBar, 1);
+		dhgenBar->show();
+		dhgen->start(QThread::LowestPriority);
+		connect(dhgen, SIGNAL(finished()),
+			this, SLOT(generateDHparamDone()));
 	} catch (errorEx &err) {
 		Error(err);
 	}
-	if (dh)
-		DH_free(dh);
-	if (fp)
-		fclose(fp);
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-	if (cb)
-		BN_GENCB_free(cb);
-#endif
-	if (bar)
-		delete bar;
 }
 
 void MainWindow::changeEvent(QEvent *event)
 {
 	if (event->type() == QEvent::LanguageChange) {
-		QList<db_base*> models;
 		retranslateUi(this);
 		dn_translations_setup();
 		init_menu();
 		update_history_menu();
-		models << keys << reqs << certs << crls << temps;
-		foreach(db_base *model, models) {
-			if (model)
-				model->updateHeaders();
-		}
+		foreach(db_base *model, models)
+			model->updateHeaders();
+
 		if (!currentDB.isEmpty())
 			dbindex->setText(tr("Database") + ": " + currentDB);
 		searchEdit->setPlaceholderText(tr("Search"));
