@@ -54,6 +54,7 @@ class tipMenu : public QMenu
 	}
 };
 
+class DHgen;
 class MainWindow: public QMainWindow, public Ui::MainWindow
 {
 	Q_OBJECT
@@ -76,7 +77,10 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void checkDB();
 		QSqlError initSqlDB();
 		QString openSqlDB(QString dbName);
-		QTimer *eachSecond;
+		QList<db_base*> models;
+		QProgressBar *dhgenBar;
+		DHgen *dhgen;
+		const QList<QStringList> getTranslators() const;
 
 	protected:
 		void init_images();
@@ -87,6 +91,8 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		QString homedir;
 		int changeDB(QString fname);
 		void keyPressEvent(QKeyEvent *e);
+		int dbTimer;
+		void timerEvent(QTimerEvent *event);
 
 	public:
 		static db_x509 *certs;
@@ -156,6 +162,7 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void exportIndex();
 		void exportIndexHierarchy();
 		void openRemoteSqlDB();
+		void generateDHparamDone();
 
 	protected slots:
 		void closeEvent(QCloseEvent * event);
